local A = unpack(select(2, ...));

A.Elements = {};

A.Elements.AddTooltip = function(frame, text, anchor)
    frame:HookScript('OnEnter', function()
        GameTooltip:SetOwner(frame, anchor or 'ANCHOR_RIGHT');
        GameTooltip:AddLine(text, 1, 0.85, 0, true);
        GameTooltip:Show();
	end);

    frame:HookScript('OnLeave', GameTooltip_Hide);
end

local PerfectBorders = 768 / string.match(C_CVar.GetCVar('gxWindowedResolution'), '%d+x(%d+)');
local PerfectScale = function(scaleObject)
    return PerfectBorders * math.floor(scaleObject / PerfectBorders);
end

function A.Elements.CreateBackdropAndBorder(frame)
    if not frame or not frame.SetBackdrop then
        return;
    end

	frame:SetBackdrop({
		bgFile   = 'Interface\\Buttons\\WHITE8x8',
        edgeFile = 'Interface\\Buttons\\WHITE8x8',
        insets   = {top = 1, left = 1, bottom = 1, right = 1},
		edgeSize = PerfectBorders + PerfectScale(1),
	});

	frame:SetBackdropColor(0.1, 0.1, 0.1, 1);
	frame:SetBackdropBorderColor(0.3, 0.3, 0.3, 1);
end

function A.Elements.CreateEditbox(name, parent, position, value, width)
    local editbox = CreateFrame('EditBox', 'ST_Emoji_'..name..'_EditBox', parent, 'InputBoxTemplate');

    editbox:ClearAllPoints();
    PixelUtil.SetPoint(editbox, unpack(position));
    PixelUtil.SetSize(editbox, width or 30, 30);
    editbox:SetFontObject(SystemFont_Shadow_Med1);
    editbox:SetText(value or '');
    editbox:SetAutoFocus(false);

    editbox.Left:Hide();
    editbox.Middle:Hide();
    editbox.Right:Hide();

    editbox.Background = CreateFrame('Frame', '$parentBackground', editbox);
    PixelUtil.SetPoint(editbox.Background, 'TOPLEFT', editbox, 'TOPLEFT', -4, -4);
    PixelUtil.SetSize(editbox.Background, width or 32, 20);
    editbox.Background:SetFrameLevel(editbox:GetFrameLevel() - 1);

    A.Elements.CreateBackdropAndBorder(editbox.Background);

    editbox:SetScript('OnEnterPressed', function(self)
        self:ClearFocus();
    end);

    editbox:SetScript('OnEscapePressed', function(self)
        self:ClearFocus();
        self:SetText('');
    end);

    editbox:SetScript('OnEditFocusGained', function(self)
        self:HighlightText();
        self.Background:SetBackdropBorderColor(0.8, 0.8, 0.8, 1);
    end);

    editbox:SetScript('OnEditFocusLost', function(self)
        EditBox_ClearHighlight(self);
        self.Background:SetBackdropBorderColor(0.3, 0.3, 0.3, 1);
    end);

    return editbox;
end

function A.Elements.CreateScrollFrame(parent, scrollChild, scrollStep)
    local scrollArea = CreateFrame('ScrollFrame', '$parentScrollFrame', parent, 'UIPanelScrollFrameTemplate');
    PixelUtil.SetPoint(scrollArea, 'TOPLEFT', parent, 'TOPLEFT', 6, -6);
    PixelUtil.SetPoint(scrollArea, 'BOTTOMRIGHT', parent, 'BOTTOMRIGHT', -27, 5);

    scrollArea:SetScrollChild(scrollChild);

    PixelUtil.SetSize(scrollChild, scrollArea:GetSize());

    local scrollbarName    = scrollArea:GetName();
    local scrollBar        = _G[scrollbarName .. 'ScrollBar'];
    local scrollUpButton   = _G[scrollbarName .. 'ScrollBarScrollUpButton'];
    local scrollDownButton = _G[scrollbarName .. 'ScrollBarScrollDownButton'];
    local scrollThumb      = _G[scrollbarName .. 'ScrollBarThumbTexture'];

    PixelUtil.SetPoint(scrollBar, 'TOPLEFT', scrollArea, 'TOPRIGHT', 6, 0);
    PixelUtil.SetPoint(scrollBar, 'BOTTOMLEFT', scrollArea, 'BOTTOMRIGHT', 6, 0);

    scrollBar.scrollStep = scrollStep;

    scrollUpButton:SetSize(1, 1);
    scrollUpButton:SetAlpha(0);

    scrollDownButton:SetSize(1, 1);
    scrollDownButton:SetAlpha(0);

    scrollThumb:SetTexture('Interface\\Buttons\\WHITE8x8');
    scrollThumb:SetSize(2, 64);
    scrollThumb:SetVertexColor(0.2, 0.2, 0.2, 1);

    local isMouseIsDown = false;
    local isMouseIsOver = false;

    scrollBar:HookScript('OnEnter', function()
        isMouseIsOver = true;
        C_Timer.After(0.25, function()
            if scrollBar:IsMouseOver() then
                scrollThumb:SetSize(6, 64)
                scrollThumb:SetVertexColor(0.3, 0.3, 0.3, 1);
            end
        end);
    end)

    scrollBar:HookScript('OnLeave', function()
        isMouseIsOver = false;
        if not isMouseIsDown then
            C_Timer.After(0.25, function()
                if not scrollBar:IsMouseOver() then
                    scrollThumb:SetSize(2, 64);
                    scrollThumb:SetVertexColor(0.2, 0.2, 0.2, 1);
                end
            end);
        end
    end)

    scrollBar:HookScript('OnMouseDown', function()
        isMouseIsDown = true;
        scrollThumb:SetSize(6, 64)
        scrollThumb:SetVertexColor(0.3, 0.3, 0.3, 1);
    end);

    scrollBar:HookScript('OnMouseUp', function()
        isMouseIsDown = false;
        if not isMouseIsOver then
            C_Timer.After(0.25, function()
                scrollThumb:SetSize(2, 64);
                scrollThumb:SetVertexColor(0.2, 0.2, 0.2, 1);
            end);
        end
    end)

    scrollBar:EnableMouse(true)
    scrollBar:HookScript('OnMouseWheel', function(self, value, scrollBar)
        ScrollFrameTemplate_OnMouseWheel(self, value, self);
    end);

    return scrollArea;
end