local A = unpack(select(2, ...));

-- Lua API
local ipairs, pairs, string_find, string_lower, table_wipe, table_sort, table_insert = ipairs, pairs, string.find, string.lower, table.wipe, table.sort, table.insert;

local EmojiListFrameSearchBox, EmojiListFrame, EmojiButton, EmojiTabsFrame;

local currentTabSelected = false;
local clearedByTab = false;

local Tabs = {
    [1] = {
        name = 'recent',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\1f553',
    },
    [2] = {
        name = 'people',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\1f600',
    },
    [3] = {
        name = 'nature',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\1f343',
    },
    [4] = {
        name = 'food',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\1f351',
    },
    [5] = {
        name = 'activities',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\1f3b2',
    },
    [6] = {
        name = 'travel',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\1f68a',
    },
    [7] = {
        name = 'objects',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\1f6c1',
    },
    [8] = {
        name = 'symbols',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\2764',
    },
    [9] = {
        name = 'flags',
        icon = 'Interface\\AddOns\\Emoji\\Media\\Discord\\1f3f3-fe0f-200d-1f308',
    },
};

EmojiListFrame = CreateFrame('Frame', 'ST_EmojiListFrame', UIParent);
EmojiListFrame:SetPoint('TOPRIGHT', ChatFrame1Background, 'TOPRIGHT', 0, 24);
EmojiListFrame:SetPoint('BOTTOMRIGHT', ChatFrame1Background, 'BOTTOMRIGHT', 0, 24);
EmojiListFrame:SetWidth(276);

A.Elements.CreateBackdropAndBorder(EmojiListFrame);

EmojiListFrame:EnableKeyboard(true);
EmojiListFrame:SetPropagateKeyboardInput(true);
EmojiListFrame:SetScript('OnKeyDown', function(self, key)
    self:SetPropagateKeyboardInput(true);
    if GetBindingFromClick(key) == 'TOGGLEGAMEMENU' then
        self:Hide();

        EmojiListFrameSearchBox:SetText('');
        EmojiButton.icon:SetDesaturated(true);

        self:SetPropagateKeyboardInput(false);
    end
end);

EmojiListFrame:Hide();

EmojiListFrameSearchBox = A.Elements.CreateEditbox('SearchBox', EmojiListFrame, {'BOTTOMLEFT', EmojiListFrame, 'TOPLEFT', 4, -7}, nil, 276);
EmojiListFrameSearchBox:SetFrameLevel(EmojiListFrame:GetFrameLevel() + 10);

EmojiListFrameSearchBox.SearchText = EmojiListFrameSearchBox:CreateFontString(nil, 'ARTWORK', 'SystemFont_Shadow_Med1');
EmojiListFrameSearchBox.SearchText:SetPoint('LEFT', EmojiListFrameSearchBox, 'LEFT', 4, 0);
EmojiListFrameSearchBox.SearchText:SetText(SEARCH);

EmojiListFrameSearchBox:SetScript('OnTextChanged', function(self)
    if not clearedByTab then
        currentTabSelected = false;
        for name, _ in pairs(EmojiTabsFrame.buttons) do
            EmojiTabsFrame.buttons[name]:SetChecked(false);
        end
    end

    EmojiListFrameSearchBox.SearchText:SetShown(self:GetText() == '');

    A.GenerateEmojiListFrame();
end);

EmojiListFrameSearchBox:SetScript('OnEscapePressed', function(self)
    clearedByTab = false;

    self:ClearFocus();
    self:SetText('');

    currentTabSelected = false;
    for name, _ in pairs(EmojiTabsFrame.buttons) do
        EmojiTabsFrame.buttons[name]:SetChecked(false);
    end

    A.GenerateEmojiListFrame();
end);

EmojiListFrameSearchBox:HookScript('OnEditFocusGained', function(self)
    EmojiListFrameSearchBox.SearchText:SetShown(self:GetText() == '');
end);

EmojiListFrameSearchBox:HookScript('OnEditFocusLost', function(self)
    EmojiListFrameSearchBox.SearchText:SetShown(self:GetText() == '');
end);


EmojiButton = CreateFrame('Button', nil, UIParent);
EmojiButton:SetPoint('RIGHT', ChatFrame1EditBox, 'RIGHT', -4, -0.5);
EmojiButton:SetSize(19, 19);
EmojiButton:SetFrameStrata('FULLSCREEN');

EmojiButton.icon = EmojiButton:CreateTexture();
EmojiButton.icon:SetAllPoints();
EmojiButton.icon:SetTexture('Interface\\AddOns\\Emoji\\Media\\Discord\\1f600');
EmojiButton.icon:SetDesaturated(true);

EmojiButton:SetScript('OnEnter', function(self)
    self.icon:SetDesaturated(false);
end);

EmojiButton:SetScript('OnLeave', function(self)
    if not EmojiListFrame:IsShown() then
        self.icon:SetDesaturated(true);
    end
end);

EmojiButton:SetScript('OnClick', function()
    EmojiListFrame:SetShown(not EmojiListFrame:IsShown());
end);

EmojiListFrame.ScrollArea = CreateFrame('Frame', nil, EmojiListFrame);
EmojiListFrame.ScrollArea.buttons = {};

A.Elements.CreateScrollFrame(EmojiListFrame, EmojiListFrame.ScrollArea, 24);

EmojiTabsFrame = CreateFrame('Frame', nil, EmojiListFrame);
EmojiTabsFrame:SetPoint('TOPLEFT', EmojiListFrame, 'BOTTOMLEFT', 0, 1);
EmojiTabsFrame:SetPoint('TOPRIGHT', EmojiListFrame, 'BOTTOMRIGHT', 0, 0);
EmojiTabsFrame:SetHeight(24);

A.Elements.CreateBackdropAndBorder(EmojiTabsFrame);

EmojiTabsFrame.buttons = {};

local function searchFind(str, needle)
    if not str then
        return;
    end

    if string_find(string_lower(str), needle) then
        return true;
    end

    return false;
end

A.GenerateTabs = function()
    for i, data in ipairs(Tabs) do
        if not EmojiTabsFrame.buttons[data.name] then
            EmojiTabsFrame.buttons[data.name] = CreateFrame('CheckButton', nil, EmojiTabsFrame);
            PixelUtil.SetPoint(EmojiTabsFrame.buttons[data.name], 'LEFT', EmojiTabsFrame, 'LEFT', ((i - 1) * 30) + 10, 0);
            PixelUtil.SetSize(EmojiTabsFrame.buttons[data.name], 17, 17);

            EmojiTabsFrame.buttons[data.name]:SetNormalTexture(data.icon);
            EmojiTabsFrame.buttons[data.name]:GetNormalTexture(data.icon):SetDesaturated(true);

            EmojiTabsFrame.buttons[data.name]:SetCheckedTexture(data.icon);
            EmojiTabsFrame.buttons[data.name]:GetCheckedTexture():SetBlendMode('BLEND')

            EmojiTabsFrame.buttons[data.name]:SetScript('OnClick', function(self)
                EmojiListFrameSearchBox:ClearFocus();
                EmojiListFrameSearchBox:SetText('');

                clearedByTab = true;

                currentTabSelected = self:GetChecked() and data.name or false;

                for name, _ in pairs(EmojiTabsFrame.buttons) do
                    if name ~= data.name then
                        EmojiTabsFrame.buttons[name]:SetChecked(false);
                    end
                end

                A.GenerateEmojiListFrame();
            end);
        end

        EmojiTabsFrame.buttons[data.name]:Show();
    end
end

local sortedRecentEmojis = {};

local function sortByCount(a, b)
	return a.count > b.count;
end

A.GenerateEmojiListFrame = function()
    for _, b in pairs(EmojiListFrame.ScrollArea.buttons) do
        b:Hide();
    end

    local search = EmojiListFrameSearchBox:GetText();

    if search and search ~= '' then
        search = string_lower(search);
        currentTabSelected = false;
    end

    local numrows, lastrowitem, numitems, lastitem = 0, nil, 0, nil;

    if currentTabSelected == 'recent' then
        table_wipe(sortedRecentEmojis);

        for _, v in pairs(EmojiDB.recent) do
            table_insert(sortedRecentEmojis, v);
        end

        table_sort(sortedRecentEmojis, sortByCount);

        for _, emoji in ipairs(sortedRecentEmojis) do
            if not EmojiListFrame.ScrollArea.buttons[emoji.file] then
                EmojiListFrame.ScrollArea.buttons[emoji.file] = CreateFrame('Button', nil, EmojiListFrame.ScrollArea);
                PixelUtil.SetSize(EmojiListFrame.ScrollArea.buttons[emoji.file], 19, 19);
            end

            EmojiListFrame.ScrollArea.buttons[emoji.file]:ClearAllPoints();

            if not lastitem then
                PixelUtil.SetPoint(EmojiListFrame.ScrollArea.buttons[emoji.file], 'TOPLEFT', EmojiListFrame.ScrollArea, 'TOPLEFT', 0, 0);
                lastrowitem = EmojiListFrame.ScrollArea.buttons[emoji.file];
            else
                if numitems == 10 then
                    PixelUtil.SetPoint(EmojiListFrame.ScrollArea.buttons[emoji.file], 'TOP', lastrowitem, 'BOTTOM', 0, -5);

                    lastrowitem = EmojiListFrame.ScrollArea.buttons[emoji.file];
                    numrows = numrows + 1;
                    numitems = 0;
                else
                    PixelUtil.SetPoint(EmojiListFrame.ScrollArea.buttons[emoji.file], 'LEFT', lastitem, 'RIGHT', 5, 0);
                end
            end

            numitems = numitems + 1;
            lastitem = EmojiListFrame.ScrollArea.buttons[emoji.file];

            if not EmojiListFrame.ScrollArea.buttons[emoji.file].icon then
                EmojiListFrame.ScrollArea.buttons[emoji.file].icon = EmojiListFrame.ScrollArea.buttons[emoji.file]:CreateTexture();
                EmojiListFrame.ScrollArea.buttons[emoji.file].icon:SetAllPoints();
            end

            EmojiListFrame.ScrollArea.buttons[emoji.file].icon:SetTexture(emoji.path .. '\\' .. emoji.file);

            EmojiListFrame.ScrollArea.buttons[emoji.file].name = emoji.name;

            EmojiListFrame.ScrollArea.buttons[emoji.file]:SetScript('OnClick', function(self)
                local editbox = ChatEdit_GetActiveWindow();

                if editbox then
                    editbox:Insert(self.name);
                else
                    editbox = ChatEdit_ChooseBoxForSend();
                    local chatFrame = editbox.chatFrame;

                    ChatFrame_OpenChat((editbox:GetText() or '') .. self.name, chatFrame);
                end

                if not IsShiftKeyDown() then
                    EmojiListFrame:SetShown(false);
                    EmojiButton.icon:SetDesaturated(true);
                end

                if not EmojiDB.recent[self.name] then
                    EmojiDB.recent[self.name] = { path = emoji.path, name = self.name, file = emoji.file, count = 1 };
                else
                    EmojiDB.recent[self.name].count = EmojiDB.recent[self.name].count + 1;
                end
            end);

            A.Elements.AddTooltip(EmojiListFrame.ScrollArea.buttons[emoji.file], '|T' .. emoji.path .. '\\' .. emoji.file .. ':0|t   ' .. emoji.name);

            EmojiListFrame.ScrollArea.buttons[emoji.file]:Show();
        end
    else
        for packName, pack in pairs(A.GetPacks()) do
            for _, emoji in pairs(pack.strategy) do
                if ((search and ( searchFind(emoji.names[1], search) or searchFind(emoji.names[2], search) or searchFind(emoji.names[3], search) ))
                    or (not search or search == ''))
                and ( (currentTabSelected and currentTabSelected == emoji.tab) or (not currentTabSelected) )
                then
                    if not EmojiListFrame.ScrollArea.buttons[emoji.file] then
                        EmojiListFrame.ScrollArea.buttons[emoji.file] = CreateFrame('Button', nil, EmojiListFrame.ScrollArea);
                        PixelUtil.SetSize(EmojiListFrame.ScrollArea.buttons[emoji.file], 19, 19);
                    end

                    EmojiListFrame.ScrollArea.buttons[emoji.file]:ClearAllPoints();

                    if not lastitem then
                        PixelUtil.SetPoint(EmojiListFrame.ScrollArea.buttons[emoji.file], 'TOPLEFT', EmojiListFrame.ScrollArea, 'TOPLEFT', 0, 0);
                        lastrowitem = EmojiListFrame.ScrollArea.buttons[emoji.file];
                    else
                        if numitems == 10 then
                            PixelUtil.SetPoint(EmojiListFrame.ScrollArea.buttons[emoji.file], 'TOP', lastrowitem, 'BOTTOM', 0, -5);

                            lastrowitem = EmojiListFrame.ScrollArea.buttons[emoji.file];
                            numrows = numrows + 1;
                            numitems = 0;
                        else
                            PixelUtil.SetPoint(EmojiListFrame.ScrollArea.buttons[emoji.file], 'LEFT', lastitem, 'RIGHT', 5, 0);
                        end
                    end

                    numitems = numitems + 1;
                    lastitem = EmojiListFrame.ScrollArea.buttons[emoji.file];

                    if not EmojiListFrame.ScrollArea.buttons[emoji.file].icon then
                        EmojiListFrame.ScrollArea.buttons[emoji.file].icon = EmojiListFrame.ScrollArea.buttons[emoji.file]:CreateTexture();
                        EmojiListFrame.ScrollArea.buttons[emoji.file].icon:SetAllPoints();
                    end

                    EmojiListFrame.ScrollArea.buttons[emoji.file].icon:SetTexture(pack.path .. '\\' .. emoji.file);

                    EmojiListFrame.ScrollArea.buttons[emoji.file].name = emoji.names[1];

                    EmojiListFrame.ScrollArea.buttons[emoji.file]:SetScript('OnClick', function(self)
                        local editbox = ChatEdit_GetActiveWindow();

                        if editbox then
                            editbox:Insert(self.name);
                        else
                            editbox = ChatEdit_ChooseBoxForSend();
                            local chatFrame = editbox.chatFrame;

                            ChatFrame_OpenChat((editbox:GetText() or '') .. self.name, chatFrame);
                        end

                        if not IsShiftKeyDown() then
                            EmojiListFrame:SetShown(false);
                            EmojiButton.icon:SetDesaturated(true);
                        end

                        if not EmojiDB.recent[self.name] then
                            EmojiDB.recent[self.name] = { path = pack.path, name = self.name, file = emoji.file, count = 1 };
                        else
                            EmojiDB.recent[self.name].count = EmojiDB.recent[self.name].count + 1;
                        end
                    end);

                    A.Elements.AddTooltip(EmojiListFrame.ScrollArea.buttons[emoji.file], '|T' .. pack.path .. '\\' .. emoji.file .. ':0|t   ' .. emoji.names[1]);

                    EmojiListFrame.ScrollArea.buttons[emoji.file]:Show();
                end
            end
        end
    end
end

A:RegisterEvent('GLOBAL_MOUSE_DOWN');
function A:GLOBAL_MOUSE_DOWN()
    if EmojiListFrame:IsShown() and not EmojiListFrame:IsMouseOver() and not EmojiButton:IsMouseOver() and not EmojiListFrameSearchBox:IsMouseOver() and not EmojiTabsFrame:IsMouseOver() then
        EmojiListFrame:SetShown(false);
        EmojiButton.icon:SetDesaturated(true);
    end
end

A:RegisterEvent('PLAYER_LOGIN')
function A:PLAYER_LOGIN()
    A.GenerateEmojiListFrame();
    A.GenerateTabs();
end